package com.fr.plugin.xxx.gzjc.sso.utils;

import com.fr.log.FineLoggerFactory;
import com.fr.log.FineLoggerProvider;
import com.fr.plugin.context.PluginContexts;
import com.fr.plugin.xxx.gzjc.sso.conf.ExcelExportConfig;
import com.fr.stable.StringUtils;

/**
 * @author xxx
 * @since 2021/12/04
 */
public final class LogUtils {
    private static final String DEBUG_PREFIX = "[插件调试] ";
    private static String LOG_PREFIX = "[OAUTH2单点登陆] ";
    private static final String PLUGIN_VERSION;

    private static final FineLoggerProvider LOGGER = FineLoggerFactory.getLogger();

    static {
        String version = PluginContexts.currentContext().getMarker().getVersion();
        if (StringUtils.isNotBlank(version)) {
            PLUGIN_VERSION = "[v" + version + "] ";
        } else {
            PLUGIN_VERSION = "[unknown version] ";
        }

        LOG_PREFIX = LOG_PREFIX + PLUGIN_VERSION;
    }

    public static void setPrefix(String prefix) {
        if (prefix != null) {
            LOG_PREFIX = prefix;
        }
    }

    public static boolean isDebugEnabled() {
        return LOGGER.isDebugEnabled();
    }

    public static void debug(String s) {
        LOGGER.debug(LOG_PREFIX + s);
    }

    public static void debug(String s, Object... objects) {
        LOGGER.debug(LOG_PREFIX + s, objects);
    }

    public static void debug(String s, Throwable throwable) {
        LOGGER.debug(LOG_PREFIX + s, throwable);
    }

    public static void debug4plugin(String s) {
        if (ExcelExportConfig.getInstance().getDebugSwitch()) {
            LOGGER.error(DEBUG_PREFIX + LOG_PREFIX + s);
        } else {
            LOGGER.debug(LOG_PREFIX + s);
        }
    }

    public static void debug4plugin(String s, Object... objects) {
        if (ExcelExportConfig.getInstance().getDebugSwitch()) {
            LOGGER.error(DEBUG_PREFIX + LOG_PREFIX + s, objects);
        } else {
            LOGGER.debug(LOG_PREFIX + s, objects);
        }
    }

    public static void debug4plugin(String s, Throwable throwable) {
        if (ExcelExportConfig.getInstance().getDebugSwitch()) {
            LOGGER.error(DEBUG_PREFIX + LOG_PREFIX + s, throwable);
        } else {
            LOGGER.debug(LOG_PREFIX + s, throwable);
        }
    }


    public static boolean isInfoEnabled() {
        return LOGGER.isInfoEnabled();
    }

    public static void info(String s) {
        LOGGER.info(LOG_PREFIX + s);
    }

    public static void info(String s, Object... objects) {
        LOGGER.info(LOG_PREFIX + s, objects);
    }

    public static void warn(String s) {
        LOGGER.warn(LOG_PREFIX + s);
    }

    public static void warn(String s, Object... objects) {
        LOGGER.warn(LOG_PREFIX + s, objects);
    }

    public static void warn(String s, Throwable throwable) {
        LOGGER.warn(LOG_PREFIX + s, throwable);
    }

    public static void warn(Throwable throwable, String s, Object... objects) {
        LOGGER.warn(throwable, LOG_PREFIX + s, objects);
    }

    public static void error(String s) {
        LOGGER.error(LOG_PREFIX + s);
    }

    public static void error(String s, Object... objects) {
        LOGGER.error(LOG_PREFIX + s, objects);
    }

    public static void error(String s, Throwable throwable) {
        LOGGER.error(LOG_PREFIX + s, throwable);
    }

    public static void error(Throwable throwable, String s, Object... objects) {
        LOGGER.error(throwable, LOG_PREFIX + s, objects);
    }
}
